@@ -1,10 +1,12 @@
-/* Copyright (c) 1991-2002 Doshita Lab. Speech Group, Kyoto University */
-/* Copyright (c) 2000-2002 Speech and Acoustics Processing Lab., NAIST */
-/*   All rights reserved   */
+/*
+ * Copyright (c) 1991-2003 Kyoto University
+ * Copyright (c) 2000-2003 NAIST
+ * All rights reserved
+ */
 
 /* m_options.c --- process options and set values */
 
-/* $Id: m_options.c,v 1.14 2003/01/06 16:10:39 ri Exp $ */
+/* $Id: m_options.c,v 1.19 2003/10/02 03:05:02 ri Exp $ */
 
 #include <julius.h>
 
@@ -219,10 +221,10 @@ opt_parse(int argc, char *argv[], char *cwd)
     } else if (strmatch(argv[i],"-tailmargin")) { /* tail silence length */
       tail_margin_msec = atoi(NEXTARG);
       continue;
-    } else if (strmatch(argv[i],"-hipass")) { /* frequency of hi cut-off */
+    } else if (strmatch(argv[i],"-hipass")||strmatch(argv[i],"-hifreq")) { /* frequency of upper band limit */
       hipass = atoi(NEXTARG);
       continue;
-    } else if (strmatch(argv[i],"-lopass")) { /* frequency of low cut-off */
+    } else if (strmatch(argv[i],"-lopass")||strmatch(argv[i],"-lofreq")) { /* frequency of lower band limit */
       lopass = atoi(NEXTARG);
       continue;
     } else if (strmatch(argv[i],"-smpPeriod")) { /* sample period (ns) */
@@ -248,11 +250,7 @@ opt_parse(int argc, char *argv[], char *cwd)
     } else if (strmatch(argv[i],"-ssfloor")) { /* spectral floor for SS */
       ssfloor = atof(NEXTARG);
       continue;
-    } else if (strmatch(argv[i],"-version")) { /* print version and exit */
-      put_header(stderr);
-      put_compile_defs(stderr);
-      j_exit();
-    } else if (strmatch(argv[i],"--version")) {
+    } else if (strmatch(argv[i],"-version") || strmatch(argv[i], "--version") || strmatch(argv[i], "-setting") || strmatch(argv[i], "--setting")) { /* print version and exit */
       put_header(stderr);
       put_compile_defs(stderr);
       j_exit();
@@ -345,14 +343,6 @@ opt_parse(int argc, char *argv[], char *cwd)
     } else if (strmatch(argv[i],"-iwspentry")) { /* content of the iwspword */
       iwspentry = NEXTARG;
       continue;
-#ifdef CLASS_NGRAM
-    } else if (strmatch(argv[i],"-class")) {
-      class_bingram_file = filepath(NEXTARG, cwd);
-      continue;
-    } else if (strmatch(argv[i],"-clw")) {
-      class_weight = atof(NEXTARG);
-      continue;
-#endif
     } else if (strmatch(argv[i],"-iwcache")) { /* control cross-word LM cache */
 #ifdef HASH_CACHE_IW
       iw_cache_rate = atof(NEXTARG);
@@ -396,6 +386,8 @@ opt_parse(int argc, char *argv[], char *cwd)
 	gprune_method = GPRUNE_SEL_BEAM;
       } else if (strmatch(tmparg,"none")) { /* no prune: compute all Gaussian */
 	gprune_method = GPRUNE_SEL_NONE;
+      } else if (strmatch(tmparg,"default")) {
+	gprune_method = GPRUNE_SEL_UNDEF;
       } else {
 	j_printerr("%s: no such pruning method \"%s\"\n", argv[0], tmparg);
 	opt_terminate();
@@ -462,6 +454,19 @@ opt_parse(int argc, char *argv[], char *cwd)
       ssload_filename = filepath(NEXTARG, cwd);
       sscalc = FALSE;
       continue;
+#ifdef CONFIDENCE_MEASURE
+    } else if (strmatch(argv[i],"-cmalpha")) { /* CM log score scaling factor */
+#ifdef CM_MULTIPLE_ALPHA
+      cm_alpha_bgn = (LOGPROB)atof(NEXTARG);
+      cm_alpha_end = (LOGPROB)atof(NEXTARG);
+      cm_alpha_step = (LOGPROB)atof(NEXTARG);
+      cm_alpha_num = (int)((cm_alpha_end - cm_alpha_bgn) / cm_alpha_step) + 1;
+      if (cm_alpha_num > 100) j_error("too small step or wide range! outputnum > 100\n");
+#else
+      cm_alpha = (LOGPROB)atof(NEXTARG);
+#endif
+      continue;
+#endif /* CONFIDENCE_MEASURE */
     } else if (strmatch(argv[i],"-help")) { /* output version and option */
       detailed_help();
     } else if (strmatch(argv[i],"--help")) {
