@@ -1,10 +1,12 @@
-/* Copyright (c) 1991-2002 Doshita Lab. Speech Group, Kyoto University */
-/* Copyright (c) 2000-2002 Speech and Acoustics Processing Lab., NAIST */
-/*   All rights reserved   */
+/*
+ * Copyright (c) 1991-2003 Kyoto University
+ * Copyright (c) 2000-2003 NAIST
+ * All rights reserved
+ */
 
 /* adin-cut.c --- read audio input from device with skipping silence */
 
-/* $Id: adin-cut.c,v 1.9 2003/04/22 01:59:44 ri Exp $ */
+/* $Id: adin-cut.c,v 1.13 2003/10/04 12:00:57 ri Exp $ */
 
 /* use zerocross & level threshold for silence detection */
 /* process one speech segment till next silence at a call */
@@ -42,6 +44,11 @@ static int thres;		/* level threshold */
 static int noise_zerocross;	/* threshold of zerocross num in a period */
 static int nc_max;		/* tail margin */
 
+/* for delayed tail silence processing */
+static SP16 *swapbuf;		/* swap buffer */
+static int sbsize, sblen;	/* size and current length of swapbuf[] */
+static int rest_tail;
+
 /* device-dependent configuration */
 static boolean adin_cut_on;	/* TRUE if cutting enabled */
 static boolean strip_flag;
@@ -97,10 +104,15 @@ adin_setup_param(int silence_cut, boolean strip_zero, int cthres, int czc, int h
   thres = cthres;
   /* calc & set internal parameter from configuration */
   samples_in_msec = (float) sample_freq / 1000.0;
+  /* cycle buffer length = head margin length */
   c_length = head_margin * samples_in_msec;	/* in msec. */
+  /* compute zerocross trigger count threshold in the cycle buffer */
   noise_zerocross = czc * c_length / sample_freq;
+  /* process step */
   wstep = DEFAULT_WSTEP;
+  /* tail margin length (in wstep) */
   nc_max = (tail_margin * samples_in_msec / wstep) + 2;
+  sbsize = tail_margin * samples_in_msec + (c_length * czc / 200);
   /*printf("c_length=%d, noise_zerocross=%d nc_max=%d\n", c_length, noise_zerocross, nc_max);*/
 
 #ifdef HAVE_PTHREAD
@@ -140,7 +152,7 @@ static SP16 *buffer = NULL;	/* buffer */
 static int bpmax;		/* maximum size of buffer */
 static int bp;			/* current point to store data */
 static int current_len;		/* current length of stored samples */
-static SP16 *cbuf;
+static SP16 *cbuf;		/* for cycle buffer flushing */
 
 /* purge processed samples */
 static void
@@ -194,6 +206,8 @@ adin_cut(int (*ad_process)(SP16 *, int), int (*ad_check)())
     end_of_stream = FALSE;
     nc = 0;
     cbuf = (SP16 *)mymalloc(sizeof(SP16) * c_length);
+    swapbuf = (SP16 *)mymalloc(sizeof(SP16) * sbsize);
+    sblen = 0;
   }
       
   /* resume input */
@@ -286,9 +300,9 @@ adin_cut(int (*ad_process)(SP16 *, int), int (*ad_check)())
 	/* count zero-cross */
 	zc = count_zc_e(&(buffer[i]), wstep);
 	if (zc > noise_zerocross) { /* triggering */
-	  nc = 0;
 	  if (is_valid_data == FALSE) {	/* triggered */
 	    is_valid_data = TRUE;   /* start to record */
+	    nc = 0;
 #ifdef THREAD_DEBUG
 	    printf("detect on\n");
 #endif
@@ -329,22 +343,85 @@ adin_cut(int (*ad_process)(SP16 *, int), int (*ad_check)())
 		}
 	      }
 	    }
-	  }
+	  } else {
+	    if (nc > 0) {
+	      /* re-triggering in tail margin */
+#ifdef THREAD_DEBUG
+	      printf("re-triggered\n");
+#endif
+	      nc = 0;
+	      /* process swap buffer stored while tail silence */
+	      if (sblen > 0) {
+#ifdef THREAD_DEBUG
+		printf("callback for swapped %d samples\n", sblen);
+#endif
+		ad_process_ret = (*ad_process)(swapbuf, sblen);
+		sblen = 0;
+		switch(ad_process_ret) {
+		case 1:		/* segmented */
+#ifdef HAVE_PTHREAD
+		  if (enable_thread) { /* just stop transfer */
+		    pthread_mutex_lock(&mutex);
+		    transfer_online = transfer_online_local = FALSE;
+		    pthread_mutex_unlock(&mutex);
+		  } else {
+		    end_status = 1;
+		    adin_purge(i);
+		    goto break_input;
+		  }
+		  break;
+#else
+		  end_status = 1;
+		  adin_purge(i);
+		  goto break_input;
+#endif
+		case -1:		/* error */
+		  end_status = -1;
+		  goto break_input;
+		}
+	      }
+	    }
+	  } 
 	} else if (is_valid_data == TRUE) {
 	  /* processing tailing silence */
 #ifdef THREAD_DEBUG
-	  printf("trailing silence\n");
+	  printf("TRAILING SILENCE\n");
 #endif
+	  if (nc == 0) {	/* start of tail silence */
+	    rest_tail = sbsize - c_length;
+	    sblen = 0;
+#ifdef THREAD_DEBUG
+	    printf("start tail silence, rest_tail = %d\n", rest_tail);
+#endif
+	  }
 	  nc++;
 	}
       }
-      if(
+      /* process data */
+      if (adin_cut_on && is_valid_data && nc > 0 && rest_tail == 0) {
+	/* store data over the tail margin to swap buffer */
+#ifdef THREAD_DEBUG
+	printf("tail silence over, store to swap buffer (nc=%d, rest_tail=%d, sblen=%d-%d)\n", nc, rest_tail, sblen, sblen+wstep);
+#endif
+	if (sblen + wstep > sbsize) {
+	  j_printerr("Error: swapbuf exceeded!\n");
+	}
+	memcpy(&(swapbuf[sblen]), &(buffer[i]), wstep * sizeof(SP16));
+	sblen += wstep;
+      } else {
+       if(
 	 (!adin_cut_on || is_valid_data == TRUE)
 #ifdef HAVE_PTHREAD
 	 && (!enable_thread || transfer_online_local)
 #endif
 	 ) {
-	/* process data */
+	if (nc > 0) {
+	  if (rest_tail < wstep) rest_tail = 0;
+	  else rest_tail -= wstep;
+#ifdef THREAD_DEBUG
+	  printf("%d processed, rest_tail=%d\n", wstep, rest_tail);
+#endif
+	}
 	if ( ad_process != NULL ) {
 #ifdef THREAD_DEBUG
 	  printf("callback for input sample [%d-%d]\n", i, i+wstep);
@@ -374,6 +451,7 @@ adin_cut(int (*ad_process)(SP16 *, int), int (*ad_check)())
 	    goto break_input;
 	  }
 	}
+       }
       }
       if (adin_cut_on && is_valid_data && nc >= nc_max) {
 #ifdef THREAD_DEBUG
@@ -381,6 +459,7 @@ adin_cut(int (*ad_process)(SP16 *, int), int (*ad_check)())
 #endif
 	/* end input by silence */
 	is_valid_data = FALSE;
+	sblen = 0;
 #ifdef HAVE_PTHREAD
 	if (enable_thread) { /* just stop transfer */
 	  pthread_mutex_lock(&mutex);
